@@ -2,7 +2,7 @@
  * File name: Rk.h
  * Project: Redkite (A small GUI toolkit)
  *
- * Copyright (C) 2019 Iurie Nistor <http://geontime.com>
+ * Copyright (C) 2019 Iurie Nistor <http://iuriepage.wordpress.com>
  *
  * This file is part of Redkite.
  *
@@ -24,10 +24,10 @@
 #ifndef RK_GLOBAL_H
 #define RK_GLOBAL_H
 
-#define RK_VERSION 0x000801
-#define RK_MAJOR   0x00
-#define RK_MINOR   0x08
-#define RK_PATCH   0x01
+#define RK_VERSION 0x010003
+#define RK_MAJOR   0x01
+#define RK_MINOR   0x00
+#define RK_PATCH   0x03
 
 #include <memory>
 #include <vector>
@@ -37,6 +37,12 @@
 #include <thread>
 #include <chrono>
 #include <functional>
+#include <unordered_map>
+#include <unordered_set>
+#include <mutex>
+#include <array>
+#include <iostream>
+#include <sstream>
 
 #define RK_UNUSED(expr) (void)expr
 
@@ -62,24 +68,24 @@
 
 #define RK_DECLARE_IMPL(Class) \
   class Class##Impl; \
-  std::shared_ptr<Class##Impl> o_ptr;
+  std::unique_ptr<Class##Impl> o_ptr
 
 #define RK_DELCATE_IMPL_PTR(Class) \
-        class Class##Impl; \
-        std::shared_ptr<Class##Impl> impl_ptr;
+  class Class##Impl; \
+  Class##Impl *impl_ptr
 
-#define RK_DECALRE_INTERFACE_PTR(Class) Class *inf_ptr;
+#define RK_DECALRE_INTERFACE_PTR(Class) Class *inf_ptr
 
-#define RK_CLASS_INFO(name, value) virtual std::string rk_property_ ##name () const { return std::string( #value ); }
-#define RK_SET_CLASS_INFO(name, value) virtual std::string rk_property_ ##name () const override { return std::string( #value ); }
+#define RK_CLASS_INFO(name, value) virtual std::string rk__property_ ##name () const { return std::string( #value ); }
+#define RK_SET_CLASS_INFO(name, value) virtual std::string rk__property_ ##name () const override { return std::string( #value ); }
 
 #define RK_DISABLE_COPY(Class) \
           Class(const Class &other) = delete; \
-          Class& operator=(const Class &other) = delete;
+          Class& operator=(const Class &other) = delete
 
 #define RK_DISABLE_MOVE(Class) \
           Class(Class &&other) = delete; \
-          Class& operator=(Class &&other) = delete; \
+          Class& operator=(Class &&other) = delete
 
 #if defined(RK_OS_WIN) && !defined(RK_FOR_SHARED)
 int rkMain(int, char **);
@@ -93,6 +99,14 @@ using rk_real = double;
 #endif // RK_SINGLE_PRECISION
 
 namespace Rk {
+        struct RkEnumClassHash {
+                template <typename T>
+                int operator()(T t) const
+                {
+                        return static_cast<int>(t);
+                }
+        };
+
         enum class Alignment : int {
                 AlignLeft   = 1,
                 AlignRight  = 2,
@@ -120,16 +134,16 @@ namespace Rk {
                 ModalParent = 1,
 
                 // Disable input for all widgets hierarhy
-                // of the top window, including top window,
+                // of the top widget, including top widget,
                 // except the current modal one.
-                ModalTopWindow = 2
+                ModalTopWidget = 2
         };
 
         enum class WidgetAttribute : int {
                 NoAttributes = 0x00000000,
                 KeyInputEnabled = 0x00000001,
                 MouseInputEnabled = 0x00000002,
-                CloseInputEnabled = 0x00000004,
+                CloseInputEnabled = 0x00000004
         };
 
         enum class Key : int {
@@ -428,36 +442,51 @@ namespace Rk {
                 Forbidden = 17,
                 Busy = 18
         };
-};
+
+        enum class ObjectType : int {
+                Object = 0,
+                Widget = 1
+        };
+}
 
 #define RK_ACT_ARGS(arg, ...) arg, ##__VA_ARGS__
 #define RK_ARG_TYPE(type, ...) type, ##__VA_ARGS__
 #define RK_ARG_VAL(val, ...) val, ##__VA_ARGS__
 
 #define RK_DECL_ACT(name, prot, type, val) \
+        class rk__observer_##name : public RkObserver { \
+        public: \
+                rk__observer_ ##name (RkObject *obj, const std::function<void(type)> &cb) \
+                        : RkObserver(obj), \
+                          observerCallback{cb} {} \
+                rk__observer_##name () = default; \
+                std::function<void(type)> observerCallback; \
+        }; \
+        \
         void prot \
         { \
-                for (const auto &cb: rk_actions_cb_ ##name ) \
-                        cb(val); \
+                for (const auto& ob: rk__observers()) {                  \
+                        auto observer = dynamic_cast<rk__observer_ ##name *>(ob.get()); \
+                        if (observer) \
+                                observer->observerCallback(val); \
+                } \
         } \
-        void rk_add_action_cb_##name (const std::function<void(type)> &cb) \
+        void rk__add_action_cb_##name (RkObject *obj, const std::function<void(type)> &cb) \
         { \
-                rk_actions_cb_##name.push_back(cb); \
-        } \
-        std::vector<std::function<void(type)>> rk_actions_cb_ ##name
+                rk__add_observer(std::make_unique<rk__observer_##name >(obj, cb)); \
+        }
 
 #define RK_ACT_BIND(obj1, act, act_args, obj2, callback) \
-        obj1->rk_add_action_cb_##act ([=](act_args){ obj2->callback; })
+        obj1->rk__add_action_cb_##act (obj2, [=](act_args){ obj2->callback; }); \
+        obj2->rk__add_bound_object(obj1)
 
 // Bind lamda functions to object actions.
-#define RK_ACT_BINDL(obj1, act, act_args, lamda) \
-        obj1->rk_add_action_cb_##act (lamda)
-
-#define RK_ACT_UNBIND_ALL(obj1, act) obj1->rk_actions_cb_##act.clear();
+#define RK_ACT_BINDL(obj1, act, act_args, lamda)   \
+        obj1->rk__add_action_cb_##act (nullptr, lamda)
 
 #define action
 
-#define RK_DECLARE_IMAGE_RC(name) extern const unsigned char rk_ ## name ## _png[]
-#define RK_IMAGE_RC(name) rk_ ## name ## _png
+#define RK_DECLARE_IMAGE_RC(name) extern const unsigned char rk__ ## name ## _png[]
+#define RK_IMAGE_RC(name) rk__ ## name ## _png
 
 #endif // RK_GLOBAL_H
